import {Config} from '../config/config';
import {Http, Response, Headers} from 'angular2/http';
import {Injectable, EventEmitter, OnInit} from 'angular2/core';
import {Observable} from 'rxjs/Rx';
import {SDLogger} from '../logger/logger';
import {Credentials} from './credentials';
import { Cookie } from 'ng2-cookies/ng2-cookies';

const API_URL = Config.UNITY_API + '/Authentication';

@Injectable()
export class AuthenticationService implements OnInit {

  public AUTH_COOKIE: string = '.unity.com';
  
  private authenticated: boolean;
  private credentials: Credentials;
  private locationWatcher = new EventEmitter();
  private expires: any = 0;
  private intervalId: any = null; 
  private expiresTimerId: any = null;
  private COOKIE_EXPIRATION_DAYS = 1;

  constructor(private http: Http, private logger: SDLogger) {
    this.credentials = this.credentials || new Credentials();

  }

  public authenticate(username: string, password: string) {
    let headers = new Headers();
    let params = JSON.stringify({ 'UserName': username, 'Password': password });

    headers.append('content-type',
      'application/json; charset=utf-8');

    this.logger.info('Calling authenticate() with ' + username + ', ' + password);

    return this.http.post(API_URL, params, {
      headers: headers
    }).map(
      (response) => {
        // get the bearer token from the response;
         this.logger.info('RESPONSE TEXT:' + JSON.stringify(response));
        this.credentials = JSON.parse(response.text());
        this.logger.info('AUTHENTICATED: ' + JSON.stringify(this.credentials));
        this.authenticated = true;
        this.logger.info('setting authenticated to ' + this.authenticated);
        this._storeCredentials(this.credentials);
        return this.credentials;
      },
      (error) => {
        this.credentials = null;
        this.logger.error('ERROR: ' + JSON.stringify(error));
      }
      ); 
  }

  public recover(email:string){
    let headers = new Headers();
    
    headers.append('content-type',
      'application/json; charset=utf-8');
    return this.http.put(API_URL + '?email=' + email, null,{
      headers: headers
    }).map( 
      (response) => {
        
        return response;
      },
      (error) => {
        this.logger.error('ERROR: ' + JSON.stringify(error));
        return error;
      }
      );
  }
  
  public getCredentials() {
    return this._getCredentials();
  }

  public logout() {
    this.credentials = new Credentials();
    this._clearCredentials();
    this.authenticated = false;
    this._emitAuthStatus(true);
  }

  public subscribe(onNext: (value: any) => void, onThrow?: (exception: any) => void, onReturn?: () => void) {
    return this.locationWatcher.subscribe(onNext, onThrow, onReturn);
  }

  public isAuthenticated() {
    this._getCredentials();
    this.logger.info('this user is ' + (this.authenticated ? '' : 'not') + ' authenticated');
    return this.authenticated;
  }

  private _getCredentials() {

    // Try to get credentials from memory. If it's not there,
    // try to load it from cookies. 
    if (this.credentials === undefined || this.credentials.access_token.length === 0) {
      var authCookie = Cookie.getCookie(this.AUTH_COOKIE);
      this.credentials = JSON.parse(authCookie);
    }
    this.credentials = this.credentials || new Credentials();
    this.authenticated = this.credentials.access_token.length > 0;

    return this.credentials;
  }

  private _emitAuthStatus(success: boolean) {
    this.locationWatcher.emit({ success: success, authenticated: this.authenticated, token: this.credentials.access_token, expires: this.expires });
  }

  private _startExpiresTimer(seconds: number) {
    if (this.expiresTimerId != null) {
      clearTimeout(this.expiresTimerId);
    }
    this.expiresTimerId = setTimeout(() => {
      console.log('Session has expired');
      //this.doLogout();
    }, seconds * 1000); // seconds * 1000
    console.log('Token expiration timer set for', seconds, "seconds");
  }

  private _clearCredentials() {

    Cookie.deleteCookie(this.AUTH_COOKIE);
  }

  private _storeCredentials(c: Credentials) {
    Cookie.setCookie(this.AUTH_COOKIE, JSON.stringify(c), this.COOKIE_EXPIRATION_DAYS, '/', this.AUTH_COOKIE);
  }

  ngOnInit() {
    this._getCredentials();
  }
} 
